@@ -166,18 +166,6 @@ func (b *Backup) Init(
 //
 //nolint:nonamedreturns
 func (b *Backup) Run(ctx context.Context, bcp *ctrl.BackupCmd, opid ctrl.OPID, l log.LogEvent) (err error) {
-	if b.brief.Sharded &&
-		b.brief.Version.IsConfigShardSupported() &&
-		util.IsSelective(bcp.Namespaces) {
-		hasConfigShard, err := topo.HasConfigShard(ctx, b.leadConn)
-		if err != nil {
-			return errors.Wrap(err, "check for Config Shard")
-		}
-		if hasConfigShard {
-			return errors.New("selective backup is not supported with Config Shard")
-		}
-	}
-
 	inf, err := topo.GetNodeInfoExt(ctx, b.nodeConn)
 	if err != nil {
 		return errors.Wrap(err, "get cluster info")
@@ -199,9 +187,17 @@ func (b *Backup) Run(ctx context.Context, bcp *ctrl.BackupCmd, opid ctrl.OPID, l
 	}
 	if v := inf.IsConfigSrv(); v {
 		rsMeta.IsConfigSvr = &v
+
+		isConfigShard, err := topo.HasConfigShard(ctx, b.leadConn)
+		if err != nil {
+			return errors.Wrap(err, "has configshard")
+		}
+		if isConfigShard {
+			rsMeta.IsConfigShard = &isConfigShard
+		}
 	}
 
-	stg, err := util.StorageFromConfig(&b.config.Storage, l)
+	stg, err := util.StorageFromConfig(&b.config.Storage, inf.Me, l)
 	if err != nil {
 		return errors.Wrap(err, "unable to get PBM storage configuration settings")
 	}
@@ -297,8 +293,14 @@ func (b *Backup) Run(ctx context.Context, bcp *ctrl.BackupCmd, opid ctrl.OPID, l
 			}
 
 			l.Debug("waiting for balancer off")
-			bs := waitForBalancerOff(ctx, b.leadConn, time.Second*30, l)
-			l.Debug("balancer status: %s", bs)
+			bs := topo.WaitForBalancerDisabled(ctx, b.leadConn, time.Second*30, l)
+			if bs.IsDisabled() {
+				l.Debug("balancer is disabled")
+			} else {
+				l.Warning("balancer is not disabled: balancer mode: %s, in balancer round: %t",
+					bs.Mode, bs.InBalancerRound)
+			}
+
 		}
 	}
 
@@ -315,7 +317,8 @@ func (b *Backup) Run(ctx context.Context, bcp *ctrl.BackupCmd, opid ctrl.OPID, l
 		}
 
 		if err := DeleteBackupFiles(stg, bcp.Name); err != nil {
-			l.Error("Failed to delete leftover files for canceled backup %q", bcpm.Name)
+			l.Error("failed to delete leftover files for canceled backup %q: %v",
+				bcpm.Name, err)
 		}
 	}()
 
@@ -382,40 +385,6 @@ func (b *Backup) Run(ctx context.Context, bcp *ctrl.BackupCmd, opid ctrl.OPID, l
 	}
 }
 
-func waitForBalancerOff(ctx context.Context, conn connect.Client, t time.Duration, l log.LogEvent) topo.BalancerMode {
-	dn := time.NewTimer(t)
-	defer dn.Stop()
-
-	tk := time.NewTicker(time.Millisecond * 500)
-	defer tk.Stop()
-
-	var bs *topo.BalancerStatus
-	var err error
-
-Loop:
-	for {
-		select {
-		case <-tk.C:
-			bs, err = topo.GetBalancerStatus(ctx, conn)
-			if err != nil {
-				l.Error("get balancer status: %v", err)
-				continue
-			}
-			if bs.Mode == topo.BalancerModeOff {
-				return topo.BalancerModeOff
-			}
-		case <-dn.C:
-			break Loop
-		}
-	}
-
-	if bs == nil {
-		return topo.BalancerMode("")
-	}
-
-	return bs.Mode
-}
-
 func (b *Backup) toState(
 	ctx context.Context,
 	status defs.Status,
@@ -681,7 +650,7 @@ func writeMeta(stg storage.Storage, meta *BackupMeta) error {
 		return errors.Wrap(err, "marshal data")
 	}
 
-	err = stg.Save(meta.Name+defs.MetadataFileSuffix, bytes.NewReader(b), -1)
+	err = stg.Save(meta.Name+defs.MetadataFileSuffix, bytes.NewReader(b))
 	return errors.Wrap(err, "write to store")
 }
 
