@@ -26,6 +26,8 @@ import (
 	"github.com/percona/percona-backup-mongodb/pbm/storage/azure"
 	"github.com/percona/percona-backup-mongodb/pbm/storage/fs"
 	"github.com/percona/percona-backup-mongodb/pbm/storage/gcs"
+	"github.com/percona/percona-backup-mongodb/pbm/storage/mio"
+	"github.com/percona/percona-backup-mongodb/pbm/storage/oss"
 	"github.com/percona/percona-backup-mongodb/pbm/storage/s3"
 	"github.com/percona/percona-backup-mongodb/pbm/topo"
 )
@@ -141,6 +143,17 @@ func (c *Config) String() string {
 			c.Storage.S3.ServerSideEncryption.SseCustomerKey = "***"
 		}
 	}
+	if c.Storage.Minio != nil {
+		if c.Storage.Minio.Credentials.AccessKeyID != "" {
+			c.Storage.Minio.Credentials.AccessKeyID = "***"
+		}
+		if c.Storage.Minio.Credentials.SecretAccessKey != "" {
+			c.Storage.Minio.Credentials.SecretAccessKey = "***"
+		}
+		if c.Storage.Minio.Credentials.SessionToken != "" {
+			c.Storage.Minio.Credentials.SessionToken = "***"
+		}
+	}
 	if c.Storage.Azure != nil {
 		if c.Storage.Azure.Credentials.Key != "" {
 			c.Storage.Azure.Credentials.Key = "***"
@@ -160,6 +173,23 @@ func (c *Config) String() string {
 			c.Storage.GCS.Credentials.HMACSecret = "***"
 		}
 	}
+	if c.Storage.OSS != nil {
+		if c.Storage.OSS.Credentials.AccessKeyID != "" {
+			c.Storage.OSS.Credentials.AccessKeyID = "***"
+		}
+		if c.Storage.OSS.Credentials.AccessKeySecret != "" {
+			c.Storage.OSS.Credentials.AccessKeySecret = "***"
+		}
+		if c.Storage.OSS.Credentials.SecurityToken != "" {
+			c.Storage.OSS.Credentials.SecurityToken = "***"
+		}
+		if c.Storage.OSS.Credentials.SessionName != "" {
+			c.Storage.OSS.Credentials.SessionName = "***"
+		}
+		if c.Storage.OSS.Credentials.RoleARN != "" {
+			c.Storage.OSS.Credentials.RoleARN = "***"
+		}
+	}
 
 	b, err := yaml.Marshal(c)
 	if err != nil {
@@ -224,9 +254,11 @@ func (cfg *PITRConf) Clone() *PITRConf {
 type StorageConf struct {
 	Type       storage.Type  `bson:"type" json:"type" yaml:"type"`
 	S3         *s3.Config    `bson:"s3,omitempty" json:"s3,omitempty" yaml:"s3,omitempty"`
+	Minio      *mio.Config   `bson:"minio,omitempty" json:"minio,omitempty" yaml:"minio,omitempty"`
 	GCS        *gcs.Config   `bson:"gcs,omitempty" json:"gcs,omitempty" yaml:"gcs,omitempty"`
 	Azure      *azure.Config `bson:"azure,omitempty" json:"azure,omitempty" yaml:"azure,omitempty"`
 	Filesystem *fs.Config    `bson:"filesystem,omitempty" json:"filesystem,omitempty" yaml:"filesystem,omitempty"`
+	OSS        *oss.Config   `bson:"oss,omitempty" json:"oss,omitempty" yaml:"oss,omitempty"`
 }
 
 func (s *StorageConf) Clone() *StorageConf {
@@ -243,10 +275,14 @@ func (s *StorageConf) Clone() *StorageConf {
 		rv.Filesystem = s.Filesystem.Clone()
 	case storage.S3:
 		rv.S3 = s.S3.Clone()
+	case storage.Minio:
+		rv.Minio = s.Minio.Clone()
 	case storage.Azure:
 		rv.Azure = s.Azure.Clone()
 	case storage.GCS:
 		rv.GCS = s.GCS.Clone()
+	case storage.OSS:
+		rv.OSS = s.OSS.Clone()
 	case storage.Blackhole: // no config
 	}
 
@@ -261,6 +297,8 @@ func (s *StorageConf) Equal(other *StorageConf) bool {
 	switch s.Type {
 	case storage.S3:
 		return s.S3.Equal(other.S3)
+	case storage.Minio:
+		return s.Minio.Equal(other.Minio)
 	case storage.Azure:
 		return s.Azure.Equal(other.Azure)
 	case storage.GCS:
@@ -286,10 +324,14 @@ func (s *StorageConf) IsSameStorage(other *StorageConf) bool {
 	switch s.Type {
 	case storage.S3:
 		return s.S3.IsSameStorage(other.S3)
+	case storage.Minio:
+		return s.Minio.IsSameStorage(other.Minio)
 	case storage.Azure:
 		return s.Azure.IsSameStorage(other.Azure)
 	case storage.GCS:
 		return s.GCS.IsSameStorage(other.GCS)
+	case storage.OSS:
+		return s.OSS.IsSameStorage(other.OSS)
 	case storage.Filesystem:
 		return s.Filesystem.IsSameStorage(other.Filesystem)
 	case storage.Blackhole:
@@ -305,10 +347,14 @@ func (s *StorageConf) Cast() error {
 		return s.Filesystem.Cast()
 	case storage.S3:
 		return s.S3.Cast()
+	case storage.Minio:
+		return s.Minio.Cast()
+	case storage.OSS:
+		return s.OSS.Cast()
+	case storage.Azure:
+		return s.Azure.Cast()
 	case storage.GCS:
-		return nil
-	case storage.Azure: // noop
-		return nil
+		return s.GCS.Cast()
 	case storage.Blackhole: // noop
 		return nil
 	}
@@ -320,10 +366,14 @@ func (s *StorageConf) Typ() string {
 	switch s.Type {
 	case storage.S3:
 		return "S3"
+	case storage.Minio:
+		return "Minio"
 	case storage.Azure:
 		return "Azure"
 	case storage.GCS:
 		return "GCS"
+	case storage.OSS:
+		return "OSS"
 	case storage.Filesystem:
 		return "FS"
 	case storage.Blackhole:
@@ -351,6 +401,19 @@ func (s *StorageConf) Path() string {
 		if s.S3.Prefix != "" {
 			path += "/" + s.S3.Prefix
 		}
+	case storage.Minio:
+		path = s.Minio.Endpoint
+		if path == "" {
+			path = "minio://" + s.Minio.Bucket
+		} else {
+			if !strings.Contains(path, "://") {
+				path = "minio://" + path
+			}
+			path += "/" + s.Minio.Bucket
+		}
+		if s.Minio.Prefix != "" {
+			path += "/" + s.Minio.Prefix
+		}
 	case storage.Azure:
 		epURL := s.Azure.EndpointURL
 		if epURL == "" {
@@ -365,6 +428,19 @@ func (s *StorageConf) Path() string {
 		if s.GCS.Prefix != "" {
 			path += "/" + s.GCS.Prefix
 		}
+	case storage.OSS:
+		path = s.OSS.EndpointURL
+		if path == "" {
+			path = "oss://" + s.OSS.Bucket
+		} else {
+			if !strings.Contains(path, "://") {
+				path = "oss://" + path
+			}
+			path += "/" + s.OSS.Bucket
+		}
+		if s.OSS.Prefix != "" {
+			path += "/" + s.OSS.Prefix
+		}
 	case storage.Filesystem:
 		path = s.Filesystem.Path
 	}
@@ -372,6 +448,21 @@ func (s *StorageConf) Path() string {
 	return path
 }
 
+func (s *StorageConf) Region() string {
+	region := ""
+
+	switch s.Type {
+	case storage.S3:
+		region = s.S3.Region
+	case storage.Minio:
+		region = s.Minio.Region
+	case storage.OSS:
+		region = s.OSS.Region
+	}
+
+	return region
+}
+
 // RestoreConf is config options for the restore
 //
 //nolint:lll
@@ -578,7 +669,10 @@ func SetConfigVar(ctx context.Context, m connect.Client, key, val string) error
 	case reflect.Int, reflect.Int32:
 		v, err = strconv.ParseInt(val, 10, 32)
 	case reflect.Int64:
-		v, err = strconv.ParseInt(val, 10, 64)
+		v, err = time.ParseDuration(val)
+		if err != nil {
+			v, err = strconv.ParseInt(val, 10, 64)
+		}
 	case reflect.Float32:
 		v, err = strconv.ParseFloat(val, 32)
 	case reflect.Float64:
