@@ -37,6 +37,13 @@ const (
 	AgentsStatusCollection = "pbmAgents"
 )
 
+const (
+	ConfigDatabasesNS      = "config.databases"
+	ConfigCollectionsNS    = "config.collections"
+	ConfigChunksNS         = "config.chunks"
+	ConfigSystemSessionsNS = "config.system.sessions"
+)
+
 const (
 	// TmpUsersCollection and TmpRoles are tmp collections used to avoid
 	// user related issues while resoring on new cluster and preserving UUID
