@@ -10,6 +10,7 @@ package oplog
 import (
 	"context"
 	"encoding/base64"
+	"encoding/hex"
 	"encoding/json"
 	"fmt"
 	"io"
@@ -81,17 +82,78 @@ var selectedNSSupportedCommands = map[string]struct{}{
 	"commitIndexBuild": {},
 }
 
+var cloningNSSupportedCommands = map[string]struct{}{
+	"create":           {},
+	"drop":             {},
+	"createIndexes":    {},
+	"deleteIndex":      {},
+	"deleteIndexes":    {},
+	"dropIndex":        {},
+	"dropIndexes":      {},
+	"commitIndexBuild": {},
+}
+
 var dontPreserveUUID = []string{
 	"admin.system.users",
 	"admin.system.roles",
 	"admin.system.keys",
+	defs.ConfigDatabasesNS,
+	defs.ConfigCollectionsNS,
+	defs.ConfigChunksNS,
 	"*.system.buckets.*", // timeseries
 	"*.system.views",     // timeseries
 }
 
+// ConfigCollToKeep defines a list of collections in the `config`
+// database that PBM will apply oplog events.
+var configCollToKeep = []string{
+	"chunks",
+	"collections",
+	"databases",
+	"shards",
+	"tags",
+	"version",
+}
+
+const (
+	settingsColl    = "settings"
+	collectionsColl = "collections"
+	chunksColl      = "chunks"
+)
+
+var settingsToSkip = []string{
+	"balancer",
+	"automerge",
+}
+
+var ErrNoCloningNamespace = errors.New("cloning namespace desn't exist")
+
+// cloneNS has all data related to cloning namespace within oplog
+type cloneNS struct {
+	snapshot.CloneNS
+	fromDB   string
+	fromColl string
+	toDB     string
+	toColl   string
+	toUUID   primitive.Binary
+}
+
+func (c *cloneNS) SetNSPair(nsPair snapshot.CloneNS) {
+	c.CloneNS = nsPair
+	c.fromDB, c.fromColl = nsPair.SplitFromNS()
+	c.toDB, c.toColl = nsPair.SplitToNS()
+}
+
+// mDBCl represents client interface for MongoDB logic used by OplogRestore
+type mDBCl interface {
+	getUUIDForNS(ctx context.Context, ns string) (primitive.Binary, error)
+	ensureCollExists(dbName string) error
+	applyOps(entries []interface{}) error
+}
+
 // OplogRestore is the oplog applyer
 type OplogRestore struct {
-	dst               *mongo.Client
+	mdb               mDBCl
 	ver               *db.Version
 	needIdxWorkaround bool
 	preserveUUIDopt   bool
@@ -120,14 +182,16 @@ type OplogRestore struct {
 
 	unsafe bool
 
-	filter OpFilter
+	filter   OpFilter
+	cloneNS  cloneNS
+	sessUUID string
 }
 
 const saveLastDistTxns = 100
 
 // NewOplogRestore creates an object for an oplog applying
 func NewOplogRestore(
-	dst *mongo.Client,
+	m *mongo.Client,
 	ic *idx.IndexCatalog,
 	sv *version.MongoVersion,
 	unsafe,
@@ -135,7 +199,7 @@ func NewOplogRestore(
 	ctxn chan phys.RestoreTxn,
 	txnErr chan error,
 ) (*OplogRestore, error) {
-	m, err := ns.NewMatcher(append(snapshot.ExcludeFromRestore, excludeFromOplog...))
+	matcher, err := ns.NewMatcher(append(snapshot.ExcludeFromRestore, excludeFromOplog...))
 	if err != nil {
 		return nil, errors.Wrap(err, "create matcher for the collections exclude")
 	}
@@ -155,13 +219,13 @@ func NewOplogRestore(
 	}
 	ver := &db.Version{v[0], v[1], v[2]}
 	return &OplogRestore{
-		dst:               dst,
+		mdb:               newMDB(m),
 		ver:               ver,
 		preserveUUIDopt:   preserveUUID,
 		preserveUUID:      preserveUUID,
 		needIdxWorkaround: needsCreateIndexWorkaround(ver),
 		indexCatalog:      ic,
-		excludeNS:         m,
+		excludeNS:         matcher,
 		noUUIDns:          noUUID,
 		txn:               ctxn,
 		txnSyncErr:        txnErr,
@@ -196,6 +260,7 @@ func (o *OplogRestore) Apply(src io.ReadCloser) (primitive.Timestamp, error) {
 	defer bsonSource.Close()
 
 	var lts primitive.Timestamp
+
 	for {
 		rawOplogEntry := bsonSource.LoadNext()
 		if rawOplogEntry == nil {
@@ -257,16 +322,50 @@ func (o *OplogRestore) SetIncludeNS(nss []string) {
 	o.includeNS = dbs
 }
 
+// SetCloneNS sets all needed data for cloning namespace:
+// collection names and target namespace UUID
+func (o *OplogRestore) SetCloneNS(ctx context.Context, ns snapshot.CloneNS) error {
+	if !ns.IsSpecified() {
+		return nil
+	}
+
+	o.cloneNS.SetNSPair(ns)
+
+	var err error
+	o.cloneNS.toUUID, err = o.mdb.getUUIDForNS(ctx, o.cloneNS.ToNS)
+	if err != nil {
+		return errors.Wrap(err, "get to ns uuid")
+	}
+	if o.cloneNS.toUUID.IsZero() {
+		return ErrNoCloningNamespace
+	}
+
+	return nil
+}
+
+// SetSessionsToExclude sets UUID for the config.system.sessions collection
+// that needs to be excluded from when oplog is applied.
+func (o *OplogRestore) SetSessionsToExclude(sessUUID string) {
+	o.sessUUID = sessUUID
+}
+
+// isOpAllowed inspects whether op is allowed from config database point of view.
+// It allows/disallows only specific collections for config database.
+// It disallows balancer settings that would cause the balancer to work during PITR.
 func isOpAllowed(oe *Record) bool {
 	coll, ok := strings.CutPrefix(oe.Namespace, "config.")
 	if !ok {
 		return true // OK: not a "config" database. allow any ops
 	}
 
-	if slices.Contains(dumprestore.ConfigCollectionsToKeep, coll) {
+	if slices.Contains(configCollToKeep, coll) {
 		return true // OK: create/update/delete a doc
 	}
 
+	if coll == settingsColl {
+		return isConfigSettingAllowed(oe)
+	}
+
 	if coll != "$cmd" || len(oe.Object) == 0 {
 		return false // other collection is not allowed
 	}
@@ -283,6 +382,134 @@ func isOpAllowed(oe *Record) bool {
 	return false
 }
 
+// isConfigSettingAllowed filter out entries from config.settings collection
+func isConfigSettingAllowed(oe *Record) bool {
+	for _, e := range oe.Query {
+		if e.Key != "_id" {
+			continue
+		}
+		setting, _ := e.Value.(string)
+		if slices.Contains(settingsToSkip, setting) {
+			return false
+		}
+	}
+	return true // allow setting from config.settings
+}
+
+// isRoutingDocExcluded checks if we need to exclude document from
+// CSRS routing tables (collections and chunks).
+// It excludes config.system.sessions related documents with specified
+// UUID (sessUUID).
+// In case whaen sessUUID is not specified, function can set sessUUID
+// based on oplog entry (op:"i") for config.system.sessions doc.
+func isRoutingDocExcluded(oe *Record, sessUUID *string) bool {
+	if !isConfigCollectionsDocAllowed(oe, sessUUID) {
+		return true
+	}
+
+	if !isConfigChunksDocAllowed(oe, *sessUUID) {
+		return true
+	}
+
+	return false
+}
+
+// isConfigCollectionsDocAllowed returns true if config.collections entry has
+// allowed document.
+// Disallowed document has:
+//   - namespace config.system.sessions,
+//   - uuid specified with sessUUID parameter,
+//   - when sessUUID is unspecified, function will try to update sessUUID
+//     from the oplog entry, and in that case sessUUID will contain updated value.
+//
+// For disallowed document false will be returned.
+func isConfigCollectionsDocAllowed(oe *Record, sessUUID *string) bool {
+	coll, ok := strings.CutPrefix(oe.Namespace, "config.")
+	if !ok {
+		return true // no config db, just skip it
+	}
+
+	if coll != collectionsColl {
+		return true
+	}
+
+	// entry is for config.collection
+	for _, e := range oe.Object {
+		if e.Key != "_id" {
+			continue
+		}
+		id, _ := e.Value.(string)
+		if id != defs.ConfigSystemSessionsNS {
+			return true
+		} else {
+			break
+		}
+	}
+
+	// try to update system.sessions from the oplog
+	for _, e := range oe.Object {
+		if e.Key != "uuid" {
+			continue
+		}
+
+		oeUUID, ok := e.Value.(primitive.Binary)
+		if !ok {
+			return true
+		}
+		uuid := hex.EncodeToString(oeUUID.Data)
+
+		if len(*sessUUID) == 0 && oe.Operation == "i" {
+			// uuid for config.system.sessions collection is created within oplog
+			*sessUUID = uuid
+			return false
+		} else {
+			return false
+		}
+	}
+
+	return true
+}
+
+// isConfigChunksDocAllowed returns true if config.chunks entry has
+// allowed document. Disallowed document has UUID of disallowed namespace
+// specified with sessUUID parameter.
+// For all other documents false will be returned.
+func isConfigChunksDocAllowed(oe *Record, sessUUID string) bool {
+	if len(sessUUID) == 0 {
+		// uuid for config.system.sessions is not set, so just allow the entry
+		return true
+	}
+
+	coll, ok := strings.CutPrefix(oe.Namespace, "config.")
+	if !ok {
+		return true // no config db, just skip it
+	}
+
+	if coll != chunksColl {
+		return true
+	}
+
+	for _, e := range oe.Object {
+		if e.Key != "uuid" {
+			continue
+		}
+
+		oeUUID, ok := e.Value.(primitive.Binary)
+		if !ok {
+			return true
+		}
+		uuid := hex.EncodeToString(oeUUID.Data)
+
+		if uuid == sessUUID {
+			return false
+		} else {
+			return true
+		}
+	}
+
+	return true
+}
+
 func (o *OplogRestore) isOpSelected(oe *Record) bool {
 	if o.includeNS == nil || o.includeNS[""] != nil {
 		return true
@@ -310,6 +537,86 @@ func (o *OplogRestore) isOpSelected(oe *Record) bool {
 	return false
 }
 
+// isOpForCloning returns whether op needs to be processed or not in case of cloning NS.
+// In case of non cloning use case, it's always true.
+func (o *OplogRestore) isOpForCloning(oe *db.Oplog) bool {
+	if !o.cloneNS.IsSpecified() {
+		return true
+	}
+
+	// i, u, d ops for cloning ns
+	if oe.Namespace == o.cloneNS.FromNS {
+		return true
+	}
+
+	// filter out all other i, u, d ops
+	if oe.Operation != "c" {
+		return false
+	}
+
+	db, coll, _ := strings.Cut(oe.Namespace, ".")
+	if coll != "$cmd" {
+		return false
+	}
+
+	cmd := oe.Object[0].Key
+	if cmd == "applyOps" {
+		return true // internal ops of applyOps are checked one by one later
+	}
+
+	if db != o.cloneNS.fromDB {
+		// it's command not relevant for db to clone from
+		return false
+	}
+
+	if _, ok := cloningNSSupportedCommands[cmd]; ok {
+		// check if command targets collection
+		collForCmd, _ := oe.Object[0].Value.(string)
+		if collForCmd == o.cloneNS.fromColl {
+			return true
+		}
+	}
+
+	return false
+}
+
+// isOpExcluded check if collection is excluded.
+// Mostly refers PBM's control collections, but also some config and admin ones.
+func (o *OplogRestore) isOpExcluded(oe *Record) bool {
+	if o.excludeNS == nil {
+		return false
+	}
+	db, coll, _ := strings.Cut(oe.Namespace, ".")
+	if coll != "$cmd" {
+		return o.excludeNS.Has(oe.Namespace)
+	}
+
+	cmd := oe.Object[0].Key
+	if cmd == "applyOps" {
+		return false // internal ops of applyOps are checked one by one later
+	}
+	if _, ok := selectedNSSupportedCommands[cmd]; ok {
+		coll, _ = oe.Object[0].Value.(string)
+		return o.excludeNS.Has(db + "." + coll)
+	}
+	// handle renameCollection and convertToCapped commands.
+	// NOTE: convertToCapped is done by creating a temporary capped collection,
+	//       inserting docs from the source collection to the collection,
+	//       and renaming it to the source collection.
+	if cmd == "renameCollection" {
+		from, _ := oe.Object[0].Value.(string)
+		if o.excludeNS.Has(from) {
+			return true
+		}
+		to, _ := oe.Object[1].Value.(string)
+		if o.excludeNS.Has(to) {
+			return true
+		}
+	}
+
+	return false
+}
+
 func (o *OplogRestore) LastOpTS() uint32 {
 	return atomic.LoadUint32(&o.lastOpT)
 }
@@ -325,11 +632,9 @@ func (o *OplogRestore) handleOp(oe db.Oplog) error {
 		return nil
 	}
 
-	if o.excludeNS.Has(oe.Namespace) {
-		return nil
-	}
-
-	if !isOpAllowed(&oe) || !o.isOpSelected(&oe) {
+	if o.isOpExcluded(&oe) || !isOpAllowed(&oe) ||
+		!o.isOpSelected(&oe) || !o.isOpForCloning(&oe) ||
+		isRoutingDocExcluded(&oe, &o.sessUUID) {
 		return nil
 	}
 
@@ -342,6 +647,31 @@ func (o *OplogRestore) handleOp(oe db.Oplog) error {
 		return nil
 	}
 
+	if err := o.setPreserveUUID(oe); err != nil {
+		return err
+	}
+
+	meta, err := txn.NewMeta(oe)
+	if err != nil {
+		return errors.Wrap(err, "get op metadata")
+	}
+
+	if meta.IsTxn() {
+		err = o.handleTxnOp(meta, oe)
+		if err != nil {
+			return errors.Wrap(err, "applying a transaction entry")
+		}
+	} else {
+		err = o.handleNonTxnOp(oe)
+		if err != nil {
+			return errors.Wrap(err, "applying an entry")
+		}
+	}
+
+	return nil
+}
+
+func (o *OplogRestore) setPreserveUUID(oe db.Oplog) error {
 	// optimization - not to parse namespace if it remains the same
 	if o.cnamespase != oe.Namespace {
 		o.preserveUUID = o.preserveUUIDopt
@@ -364,23 +694,6 @@ func (o *OplogRestore) handleOp(oe db.Oplog) error {
 		o.cnamespase = oe.Namespace
 	}
 
-	meta, err := txn.NewMeta(oe)
-	if err != nil {
-		return errors.Wrap(err, "get op metadata")
-	}
-
-	if meta.IsTxn() {
-		err = o.handleTxnOp(meta, oe)
-		if err != nil {
-			return errors.Wrap(err, "applying a transaction entry")
-		}
-	} else {
-		err = o.handleNonTxnOp(oe)
-		if err != nil {
-			return errors.Wrap(err, "applying an entry")
-		}
-	}
-
 	return nil
 }
 
@@ -645,15 +958,54 @@ func (o *OplogRestore) HandleUncommittedTxn(
 	return partial, uncommitted, nil
 }
 
+func (o *OplogRestore) cloneEntry(op *db.Oplog) {
+	if !o.cloneNS.IsSpecified() {
+		return
+	}
+
+	// op: i, u, d
+	if op.Namespace == o.cloneNS.FromNS {
+		op.UI = nil
+		op.Namespace = o.cloneNS.ToNS
+		return
+	}
+
+	if op.Operation != "c" || len(op.Object) == 0 {
+		return
+	}
+
+	dbName, _, _ := strings.Cut(op.Namespace, ".")
+	if dbName != o.cloneNS.fromDB {
+		return
+	}
+
+	cmdName := op.Object[0].Key
+	if _, ok := cloningNSSupportedCommands[cmdName]; !ok {
+		return
+	}
+
+	collName, _ := op.Object[0].Value.(string)
+	if collName != o.cloneNS.fromColl {
+		return
+	}
+
+	// op: create/drop
+	op.Namespace = fmt.Sprintf("%s.$cmd", o.cloneNS.toDB)
+	op.Object[0].Value = o.cloneNS.toColl
+	op.UI = nil
+}
+
 func (o *OplogRestore) handleNonTxnOp(op db.Oplog) error {
 	// have to handle it here one more time because before the op gets thru
 	// txnBuffer its namespace is `collection.$cmd` instead of the real one
-	if o.excludeNS.Has(op.Namespace) {
+	if o.isOpExcluded(&op) || !isOpAllowed(&op) ||
+		!o.isOpSelected(&op) || !o.isOpForCloning(&op) ||
+		isRoutingDocExcluded(&op, &o.sessUUID) {
 		return nil
 	}
 
-	if !isOpAllowed(&op) || !o.isOpSelected(&op) {
-		return nil
+	if err := o.setPreserveUUID(op); err != nil {
+		return err
 	}
 
 	op, err := o.filterUUIDs(op)
@@ -661,6 +1013,8 @@ func (o *OplogRestore) handleNonTxnOp(op db.Oplog) error {
 		return errors.Wrap(err, "filtering UUIDs from oplog")
 	}
 
+	o.cloneEntry(&op)
+
 	dbName, collName, _ := strings.Cut(op.Namespace, ".")
 	if op.Operation == "c" {
 		if len(op.Object) == 0 {
@@ -793,20 +1147,14 @@ func (o *OplogRestore) handleNonTxnOp(op db.Oplog) error {
 		}
 	} else if op.Operation == "i" && collName == "system.views" {
 		// PBM-921: ensure the collection exists before "creating" views or timeseries
-		err := o.dst.Database(dbName).CreateCollection(context.TODO(), "system.views")
+		err := o.mdb.ensureCollExists(dbName)
 		if err != nil {
-			// MongoDB 5.0 and 6.0 returns NamespaceExists error.
-			// MongoDB 7.0 and 8.0 does not return error.
-			// https://github.com/mongodb/mongo/blob/v6.0/src/mongo/base/error_codes.yml#L84
-			const NamespaceExists = 48
-			var cmdError mongo.CommandError
-			if !errors.As(err, &cmdError) || cmdError.Code != NamespaceExists {
-				return errors.Wrapf(err, "ensure %s.system.views collection", dbName)
-			}
+			return err
 		}
+
 	}
 
-	err = o.applyOps([]interface{}{op})
+	err = o.mdb.applyOps([]interface{}{op})
 	if err != nil {
 		// https://jira.percona.com/browse/PBM-818
 		if o.unsafe && op.Namespace == "config.chunks" {
@@ -908,25 +1256,6 @@ func extractIndexDocumentFromCommitIndexBuilds(op db.Oplog) (string, []*idx.Inde
 	return collectionName, nil
 }
 
-// applyOps is a wrapper for the applyOps database command, we pass in
-// a session to avoid opening a new connection for a few inserts at a time.
-func (o *OplogRestore) applyOps(entries []interface{}) error {
-	singleRes := o.dst.Database("admin").RunCommand(context.TODO(), bson.D{{"applyOps", entries}})
-	if err := singleRes.Err(); err != nil {
-		return errors.Wrap(err, "applyOps")
-	}
-	res := bson.M{}
-	err := singleRes.Decode(&res)
-	if err != nil {
-		return errors.Wrap(err, "decode singleRes")
-	}
-	if isFalsy(res["ok"]) {
-		return errors.Errorf("applyOps command: %v", res["errmsg"])
-	}
-
-	return nil
-}
-
 // filterUUIDs removes 'ui' entries from ops, including nested applyOps ops.
 // It also modifies ops that rely on 'ui'.
 func (o *OplogRestore) filterUUIDs(op db.Oplog) (db.Oplog, error) {
