@@ -20,6 +20,7 @@ import (
 	"github.com/percona/percona-backup-mongodb/pbm/log"
 	"github.com/percona/percona-backup-mongodb/pbm/oplog"
 	"github.com/percona/percona-backup-mongodb/pbm/restore/phys"
+	"github.com/percona/percona-backup-mongodb/pbm/snapshot"
 	"github.com/percona/percona-backup-mongodb/pbm/storage"
 	"github.com/percona/percona-backup-mongodb/pbm/topo"
 	"github.com/percona/percona-backup-mongodb/pbm/util"
@@ -286,11 +287,13 @@ func chunks(
 }
 
 type applyOplogOption struct {
-	start  *primitive.Timestamp
-	end    *primitive.Timestamp
-	nss    []string
-	unsafe bool
-	filter oplog.OpFilter
+	start    *primitive.Timestamp
+	end      *primitive.Timestamp
+	nss      []string
+	cloudNS  snapshot.CloneNS
+	unsafe   bool
+	filter   oplog.OpFilter
+	sessUUID string
 }
 
 type (
@@ -324,7 +327,7 @@ type (
 func applyOplog(
 	ctx context.Context,
 	node *mongo.Client,
-	chunks []oplog.OplogChunk,
+	ranges []oplogRange,
 	options *applyOplogOption,
 	sharded bool,
 	ic *idx.IndexCatalog,
@@ -332,9 +335,8 @@ func applyOplog(
 	getTxn getcommittedTxnFn,
 	stat *phys.DistTxnStat,
 	mgoV *version.MongoVersion,
-	stg storage.Storage,
-	log log.LogEvent,
 ) (partial []oplog.Txn, err error) {
+	log := log.LogEventFromContext(ctx)
 	log.Info("starting oplog replay")
 
 	var (
@@ -342,7 +344,14 @@ func applyOplog(
 		txnSyncErr chan error
 	)
 
-	oplogRestore, err := oplog.NewOplogRestore(node, ic, mgoV, options.unsafe, true, ctxn, txnSyncErr)
+	oplogRestore, err := oplog.NewOplogRestore(
+		node,
+		ic,
+		mgoV,
+		options.unsafe,
+		true,
+		ctxn,
+		txnSyncErr)
 	if err != nil {
 		return nil, errors.Wrap(err, "create oplog")
 	}
@@ -358,27 +367,38 @@ func applyOplog(
 	}
 	oplogRestore.SetTimeframe(startTS, endTS)
 	oplogRestore.SetIncludeNS(options.nss)
+	err = oplogRestore.SetCloneNS(ctx, options.cloudNS)
+	if errors.Is(err, oplog.ErrNoCloningNamespace) {
+		log.Info("cloning namespace doesn't exist so oplog will not be applied")
+		return partial, nil
+	} else if err != nil {
+		return nil, errors.Wrap(err, "set cloning ns")
+	}
+	oplogRestore.SetSessionsToExclude(options.sessUUID)
 
 	var lts primitive.Timestamp
-	for _, chnk := range chunks {
-		log.Debug("+ applying %v", chnk)
+	for _, oplogRange := range ranges {
+		stg := oplogRange.storage
+		for _, chnk := range oplogRange.chunks {
+			log.Debug("+ applying %v", chnk)
 
-		// If the compression is Snappy and it failed we try S2.
-		// Up until v1.7.0 the compression of pitr chunks was always S2.
-		// But it was a mess in the code which lead to saving pitr chunk files
-		// with the `.snappy`` extension although it was S2 in fact. And during
-		// the restore, decompression treated .snappy as S2 ¯\_(ツ)_/¯ It wasn’t
-		// an issue since there was no choice. Now, Snappy produces `.snappy` files
-		// and S2 - `.s2` which is ok. But this means the old chunks (made by previous
-		// PBM versions) won’t be compatible - during the restore, PBM will treat such
-		// files as Snappy (judging by its suffix) but in fact, they are s2 files
-		// and restore will fail with snappy: corrupt input. So we try S2 in such a case.
-		lts, err = replayChunk(chnk.FName, oplogRestore, stg, chnk.Compression)
-		if err != nil && errors.Is(err, snappy.ErrCorrupt) {
-			lts, err = replayChunk(chnk.FName, oplogRestore, stg, compress.CompressionTypeS2)
-		}
-		if err != nil {
-			return nil, errors.Wrapf(err, "replay chunk %v.%v", chnk.StartTS.T, chnk.EndTS.T)
+			// If the compression is Snappy and it failed we try S2.
+			// Up until v1.7.0 the compression of pitr chunks was always S2.
+			// But it was a mess in the code which lead to saving pitr chunk files
+			// with the `.snappy`` extension although it was S2 in fact. And during
+			// the restore, decompression treated .snappy as S2 ¯\_(ツ)_/¯ It wasn’t
+			// an issue since there was no choice. Now, Snappy produces `.snappy` files
+			// and S2 - `.s2` which is ok. But this means the old chunks (made by previous
+			// PBM versions) won’t be compatible - during the restore, PBM will treat such
+			// files as Snappy (judging by its suffix) but in fact, they are s2 files
+			// and restore will fail with snappy: corrupt input. So we try S2 in such a case.
+			lts, err = replayChunk(chnk.FName, oplogRestore, stg, chnk.Compression)
+			if err != nil && errors.Is(err, snappy.ErrCorrupt) {
+				lts, err = replayChunk(chnk.FName, oplogRestore, stg, compress.CompressionTypeS2)
+			}
+			if err != nil {
+				return nil, errors.Wrapf(err, "replay chunk %v.%v", chnk.StartTS.T, chnk.EndTS.T)
+			}
 		}
 	}
 
