@@ -8,15 +8,17 @@ import (
 
 type Blackhole struct{}
 
+var _ storage.Storage = &Blackhole{}
+
 func New() *Blackhole {
 	return &Blackhole{}
 }
 
 func (*Blackhole) Type() storage.Type {
-	return storage.BlackHole
+	return storage.Blackhole
 }
 
-func (*Blackhole) Save(_ string, data io.Reader, _ int64) error {
+func (*Blackhole) Save(_ string, data io.Reader, _ ...storage.Option) error {
 	_, err := io.Copy(io.Discard, data)
 	return err
 }
@@ -25,6 +27,7 @@ func (*Blackhole) List(_, _ string) ([]storage.FileInfo, error) { return []stora
 func (*Blackhole) Delete(_ string) error                        { return nil }
 func (*Blackhole) FileStat(_ string) (storage.FileInfo, error)  { return storage.FileInfo{}, nil }
 func (*Blackhole) Copy(_, _ string) error                       { return nil }
+func (*Blackhole) DownloadStat() storage.DownloadStat           { return storage.DownloadStat{} }
 
 // NopReadCloser is a no operation ReadCloser
 type NopReadCloser struct{}
