@@ -4,7 +4,6 @@ import (
 	"context"
 	"fmt"
 	"os"
-	"strings"
 	"testing"
 
 	gcs "cloud.google.com/go/storage"
@@ -122,64 +121,3 @@ func TestGCS(t *testing.T) {
 		}
 	})
 }
-
-func TestConfig(t *testing.T) {
-	opts := &Config{
-		Bucket: "bucketName",
-		Prefix: "prefix",
-		Credentials: Credentials{
-			ClientEmail: "email@example.com",
-			PrivateKey:  "-----BEGIN PRIVATE KEY-----\nKey\n-----END PRIVATE KEY-----\n",
-		},
-	}
-
-	t.Run("Clone", func(t *testing.T) {
-		clone := opts.Clone()
-		if clone == opts {
-			t.Error("expected clone to be a different pointer")
-		}
-
-		if !opts.Equal(clone) {
-			t.Error("expected clone to be equal")
-		}
-
-		opts.Bucket = "updatedName"
-		if opts.Equal(clone) {
-			t.Error("expected clone to be unchanged when updating original")
-		}
-	})
-
-	t.Run("Equal fails", func(t *testing.T) {
-		if opts.Equal(nil) {
-			t.Error("expected not to be equal other nil")
-		}
-
-		clone := opts.Clone()
-		clone.Prefix = "updatedPrefix"
-		if opts.Equal(clone) {
-			t.Error("expected not to be equal when updating prefix")
-		}
-
-		clone = opts.Clone()
-		clone.Credentials.ClientEmail = "updated@example.com"
-		if opts.Equal(clone) {
-			t.Error("expected not to be equal when updating credentials")
-		}
-	})
-}
-
-func TestEmptyCredentialsFail(t *testing.T) {
-	opts := &Config{
-		Bucket: "bucketName",
-	}
-
-	_, err := New(opts, "node", nil)
-
-	if err == nil {
-		t.Fatalf("expected error when not specifying credentials")
-	}
-
-	if !strings.Contains(err.Error(), "required for GCS credentials") {
-		t.Errorf("expected required credentials, got %s", err)
-	}
-}
