@@ -21,28 +21,28 @@ import (
 
 type googleClient struct {
 	bucketHandle *storagegcs.BucketHandle
-	opts         *Config
+	cfg          *Config
 	log          log.LogEvent
 }
 
-func newGoogleClient(opts *Config, l log.LogEvent) (*googleClient, error) {
+func newGoogleClient(cfg *Config, l log.LogEvent) (*googleClient, error) {
 	ctx := context.Background()
 
-	if opts.Credentials.PrivateKey == "" || opts.Credentials.ClientEmail == "" {
+	if cfg.Credentials.PrivateKey == "" || cfg.Credentials.ClientEmail == "" {
 		return nil, errors.New("clientEmail and privateKey are required for GCS credentials")
 	}
 
 	creds, err := json.Marshal(ServiceAccountCredentials{
 		Type:                "service_account",
-		PrivateKey:          opts.Credentials.PrivateKey,
-		ClientEmail:         opts.Credentials.ClientEmail,
+		PrivateKey:          cfg.Credentials.PrivateKey,
+		ClientEmail:         cfg.Credentials.ClientEmail,
 		AuthURI:             "https://accounts.google.com/o/oauth2/auth",
 		TokenURI:            "https://oauth2.googleapis.com/token",
 		UniverseDomain:      "googleapis.com",
 		AuthProviderCertURL: "https://www.googleapis.com/oauth2/v1/certs",
 		ClientCertURL: fmt.Sprintf(
 			"https://www.googleapis.com/robot/v1/metadata/x509/%s",
-			opts.Credentials.ClientEmail,
+			cfg.Credentials.ClientEmail,
 		),
 	})
 	if err != nil {
@@ -54,27 +54,42 @@ func newGoogleClient(opts *Config, l log.LogEvent) (*googleClient, error) {
 		return nil, errors.Wrap(err, "new GCS client")
 	}
 
-	bh := cli.Bucket(opts.Bucket)
-
-	if opts.Retryer != nil {
-		bh = bh.Retryer(
+	bh := cli.Bucket(cfg.Bucket).
+		Retryer(
 			storagegcs.WithBackoff(gax.Backoff{
-				Initial:    opts.Retryer.BackoffInitial,
-				Max:        opts.Retryer.BackoffMax,
-				Multiplier: opts.Retryer.BackoffMultiplier,
+				Initial:    cfg.Retryer.BackoffInitial,
+				Max:        cfg.Retryer.BackoffMax,
+				Multiplier: cfg.Retryer.BackoffMultiplier,
 			}),
-
+			storagegcs.WithMaxAttempts(cfg.Retryer.MaxAttempts),
 			storagegcs.WithPolicy(storagegcs.RetryAlways),
+			storagegcs.WithErrorFunc(shouldRetryExtended),
 		)
-	}
 
 	return &googleClient{
 		bucketHandle: bh,
-		opts:         opts,
+		cfg:          cfg,
 		log:          l,
 	}, nil
 }
 
+// shouldRetryExtended extends default shouldRetry with mainly
+// `client connection lost` error from std library's http package.
+func shouldRetryExtended(err error) bool {
+	if err == nil {
+		return false
+	}
+	if storagegcs.ShouldRetry(err) {
+		return true
+	}
+	if strings.Contains(err.Error(), "http2: client connection lost") ||
+		strings.Contains(err.Error(), "connect: network is unreachable") {
+		return true
+	}
+
+	return false
+}
+
 func (g googleClient) save(name string, data io.Reader, options ...storage.Option) error {
 	opts := storage.GetDefaultOpts()
 	for _, opt := range options {
@@ -87,10 +102,10 @@ func (g googleClient) save(name string, data io.Reader, options ...storage.Optio
 
 	partSize := storage.ComputePartSize(
 		opts.Size,
-		10<<20, // default: 10 MiB
+		defaultChunkSize,
 		align,
 		10_000,
-		int64(g.opts.ChunkSize),
+		int64(g.cfg.ChunkSize),
 	)
 
 	if rem := partSize % align; rem != 0 {
@@ -105,8 +120,9 @@ func (g googleClient) save(name string, data io.Reader, options ...storage.Optio
 	}
 
 	ctx := context.Background()
-	w := g.bucketHandle.Object(path.Join(g.opts.Prefix, name)).NewWriter(ctx)
+	w := g.bucketHandle.Object(path.Join(g.cfg.Prefix, name)).NewWriter(ctx)
 	w.ChunkSize = int(partSize)
+	w.ChunkRetryDeadline = g.cfg.Retryer.ChunkRetryDeadline
 	if g.log != nil && opts.UseLogger {
 		w.ProgressFunc = func(written int64) {
 			if opts.Size > 0 {
@@ -133,7 +149,7 @@ func (g googleClient) save(name string, data io.Reader, options ...storage.Optio
 func (g googleClient) fileStat(name string) (storage.FileInfo, error) {
 	ctx := context.Background()
 
-	attrs, err := g.bucketHandle.Object(path.Join(g.opts.Prefix, name)).Attrs(ctx)
+	attrs, err := g.bucketHandle.Object(path.Join(g.cfg.Prefix, name)).Attrs(ctx)
 	if err != nil {
 		if errors.Is(err, storagegcs.ErrObjectNotExist) {
 			return storage.FileInfo{}, storage.ErrNotExist
@@ -196,7 +212,7 @@ func (g googleClient) list(prefix, suffix string) ([]storage.FileInfo, error) {
 func (g googleClient) delete(name string) error {
 	ctx := context.Background()
 
-	err := g.bucketHandle.Object(path.Join(g.opts.Prefix, name)).Delete(ctx)
+	err := g.bucketHandle.Object(path.Join(g.cfg.Prefix, name)).Delete(ctx)
 	if err != nil {
 		if errors.Is(err, storagegcs.ErrObjectNotExist) {
 			return storage.ErrNotExist
@@ -210,8 +226,8 @@ func (g googleClient) delete(name string) error {
 func (g googleClient) copy(src, dst string) error {
 	ctx := context.Background()
 
-	srcObj := g.bucketHandle.Object(path.Join(g.opts.Prefix, src))
-	dstObj := g.bucketHandle.Object(path.Join(g.opts.Prefix, dst))
+	srcObj := g.bucketHandle.Object(path.Join(g.cfg.Prefix, src))
+	dstObj := g.bucketHandle.Object(path.Join(g.cfg.Prefix, dst))
 
 	_, err := g.fileStat(src)
 	if err == storage.ErrNotExist {
@@ -226,7 +242,7 @@ func (g googleClient) getPartialObject(name string, buf *storage.Arena, start, l
 	ctx, cancel := context.WithTimeout(context.Background(), time.Second*60)
 	defer cancel()
 
-	obj := g.bucketHandle.Object(path.Join(g.opts.Prefix, name))
+	obj := g.bucketHandle.Object(path.Join(g.cfg.Prefix, name))
 	reader, err := obj.NewRangeReader(ctx, start, length)
 	if err != nil {
 		if errors.Is(err, storagegcs.ErrObjectNotExist) || isRangeNotSatisfiable(err) {
